<?php declare(strict_types = 1);

/*
 * Simple burger order software accepting a customers name and a choice of 3 different burgers.
 * @author Thorsten 'stepo' Hallwas
 */

error_reporting (0);
use Composite\Composite\Burger\Decorator\CheeseDecorator;
use Composite\Composite\Burger\Decorator\CucumberDecorator;
use Composite\Composite\Burger\Decorator\OnionDecorator;
use Composite\Composite\Burger\Decorator\SaladDecorator;
use Composite\Composite\Burger\Decorator\TomatoDecorator;
use Composite\Composite\Burger\Strategy\BeanBurgerPattyStrategy;
use Composite\Composite\Burger\Strategy\BeefBurgerPattyStrategy;
use Composite\Composite\Burger\Strategy\ChickenBurgerPattyStrategy;
use Composite\Composite\Burger\Strategy\FishBurgerPattyStrategy;
use Composite\Composite\Burger\Strategy\SeitanBurgerPattyStrategy;
use Composite\Composite\Burger\Template\BurgerOrder;
use Composite\Composite\Drink\Template\ColaOrder;
use Composite\Composite\Drink\Template\LemonadeOrder;
use Composite\Composite\Fries\Decorator\KetchupDecorator;
use Composite\Composite\Fries\Decorator\MayonnaiseDecorator;
use Composite\Composite\Fries\Template\FrenchFriesOrder;
use Composite\Composite\MainOrder;
use Composite\Composite\Salad\Decorator\VinaigretteDecorator;
use Composite\Composite\Salad\Decorator\YogurtDressingDecorator;
use Composite\Composite\Salad\Template\SaladOrder;
use Composite\OrderInterface;

include __DIR__.'/vendor/autoload.php';
include __DIR__.'/0-framework/header.php';

$customer = $_GET['customer'] ?: '';
$burger = $_GET['burger'] ?: null;
$burgerExtras = $_GET['burgerExtras'] ?: [];
$fries = $_GET['fries'] ?: null;
$friesExtras = $_GET['friesExtras'] ?: [];
$salad = $_GET['salad'] ?: null;
$saladExtras = $_GET['saladExtras'] ?: [];
$drink = $_GET['drink'] ?: null;

if ($customer && ($burger || $fries || $salad || $drink)) {
    $order = createOrder(
        $customer,
        $burger,
        $burgerExtras,
        $fries,
        $friesExtras,
        $salad,
        $saladExtras,
        $drink
    );
    printOrderSummary($order);
} else {
    printOrderForm();
}

include __DIR__.'/0-framework/footer.php';

/**
 * Create a new order consisting of any combination of a burger, fries, a salad and a drink.
 *
 * @param string      $customer
 * @param string|null $burger
 * @param array|null  $burgerExtras
 * @param string|null $fries
 * @param array|null  $friesExtras
 * @param string|null $salad
 * @param array|null  $saladExtras
 * @param string|null $drink
 * @return OrderInterface
 */
function createOrder(
    string $customer,
    ?string $burger,
    ?array $burgerExtras,
    ?string $fries,
    ?array $friesExtras,
    ?string $salad,
    ?array $saladExtras,
    ?string $drink
): OrderInterface {
    $orders = [];

    if (is_string($burger)) {
        $orders[] = createBurgerOrder($customer, $burger, $burgerExtras);
    }

    if (is_string($salad)) {
        $orders[] = createSaladOrder($customer, $salad, $saladExtras);
    }

    if (is_string($fries)) {
        $orders[] = createFriesOrder($customer, $fries, $friesExtras);
    }

    if (is_string($drink)) {
        $orders[] = createDrinkOrder($customer, $drink);
    }

    return new MainOrder($customer, $orders);
}

/**
 * Create a new order for a drink.
 *
 * @param string $customer
 * @param string $drink
 * @return OrderInterface
 */
function createDrinkOrder(string $customer, string $drink): OrderInterface
{
    switch ($drink) {
        case 'cola':
            $drinkOrder = new ColaOrder($customer);
            break;
        default:
            $drinkOrder = new LemonadeOrder($customer);
    }

    return $drinkOrder;
}

/**
 * Create a new order of fries.
 *
 * @param string     $customer
 * @param string     $fries
 * @param array|null $friesExtras
 * @return OrderInterface
 */
function createFriesOrder(string $customer, string $fries, ?array $friesExtras): OrderInterface
{
    switch ($fries) {
        default:
            $friesOrder = new FrenchFriesOrder($customer);
    }

    if (is_array($friesExtras)) {
        foreach ($friesExtras as $friesExtra) {
            $friesOrder = addExtraToFriesOrder($friesExtra, $friesOrder);
        }
    }

    return $friesOrder;
}

/**
 * Add the specified extra to the fries.
 *
 * @param string         $extraIdentifier
 * @param OrderInterface $order
 * @return KetchupDecorator|MayonnaiseDecorator|OrderInterface
 */
function addExtraToFriesOrder(string $extraIdentifier, OrderInterface $order)
{
    switch ($extraIdentifier) {
        case 'ketchup':
            return new KetchupDecorator($order);
        case 'mayonnaise':
            return new MayonnaiseDecorator($order);
    }

    return $order;
}

/**
 * Create a new order of a salad.
 *
 * @param string     $customer
 * @param string     $salad
 * @param array|null $saladExtras
 * @return OrderInterface
 */
function createSaladOrder(string $customer, string $salad, ?array $saladExtras): OrderInterface
{
    switch ($salad) {
        default:
            $saladOrder = new SaladOrder($customer);
    }

    if (is_array($saladExtras)) {
        foreach ($saladExtras as $saladExtra) {
            $saladOrder = addExtraToSaladOrder($saladExtra, $saladOrder);
        }
    }

    return $saladOrder;
}

/**
 * Add the specified extra to the salad.
 *
 * @param string         $extraIdentifier
 * @param OrderInterface $order
 * @return OrderInterface
 */
function addExtraToSaladOrder(string $extraIdentifier, OrderInterface $order): OrderInterface
{
    switch ($extraIdentifier) {
        case 'vinaigrette':
            return new VinaigretteDecorator($order);
        case 'yogurt':
            return new YogurtDressingDecorator($order);
    }

    return $order;
}

/**
 * Create a new order for the given customer containing the specified burger.
 * @param string     $customer
 * @param string     $burger
 * @param array|null $burgerExtras
 * @return OrderInterface
 */
function createBurgerOrder(string $customer, string $burger, ?array $burgerExtras): OrderInterface
{
    switch ($burger) {
        case 'bean':
            $patty = new BeanBurgerPattyStrategy();
            break;
        case 'beef':
            $patty = new BeefBurgerPattyStrategy();
            break;
        case 'chicken':
            $patty = new ChickenBurgerPattyStrategy();
            break;
        case 'fish':
            $patty = new FishBurgerPattyStrategy();
            break;
        default:
            $patty = new SeitanBurgerPattyStrategy();
    }
    $order = new BurgerOrder($customer, $patty);
    if (is_array($burgerExtras)) {
        foreach ($burgerExtras as $burgerExtra) {
            $order = addExtraToBurgerOrder($burgerExtra, $order);
        }
    }

    return $order;
}

/**
 * Add the specified extra to the burger.
 *
 * @param string         $extraIdentifier
 * @param OrderInterface $order
 * @return OrderInterface
 */
function addExtraToBurgerOrder(string $extraIdentifier, OrderInterface $order): OrderInterface
{
    switch ($extraIdentifier) {
        case 'cheese':
            return new CheeseDecorator($order);
        case 'cucumber':
            return new CucumberDecorator($order);
        case 'onion':
            return new OnionDecorator($order);
        case 'salad':
            return new SaladDecorator($order);
        case 'tomato':
            return new TomatoDecorator($order);
    }

    return $order;
}

/**
 * Print the welcome text and the form.
 */
function printOrderForm()
{
    echo <<<HTML
    <h1>Welcome to Mega-Burger</h1>
    <h2>Please create your order:</h2>
    <form method="get">
        <div class="form-group">
            <label for="customer">Your name</label>
            <input class="form-control" id="customer" type="text" name="customer" required />
        </div><br>
        <div class="form-group">
            <label for="burger">Do you want a burger?</label>        
            <select class="form-control" id="burger" name="burger">
                <option value="" selected>No Burger</option>
                <option value="beef">Beef-Burger</option>
                <option value="bean">Bean-Burger</option>
                <option value="chicken">Chicken-Burger</option>
                <option value="fish">Fish-Burger</option>
                <option value="seitan">Seitan-Burger</option>
            </select>
        </div>
        <div class="form-group">
            <label for="burgerExtras">Do you want extras with your burger?</label><br>             
            <input type="checkbox" name="burgerExtras[]" id="burgerExtras" value="cheese"> Cheese<br>
            <input type="checkbox" name="burgerExtras[]" id="burgerExtras" value="cucumber"> Cucumber<br>
            <input type="checkbox" name="burgerExtras[]" id="burgerExtras" value="onion"> Onion<br>
            <input type="checkbox" name="burgerExtras[]" id="burgerExtras" value="salad"> Salad<br>  
            <input type="checkbox" name="burgerExtras[]" id="burgerExtras" value="tomato"> Tomato<br>
        </div><br>  
        <div class="form-group">
            <label for="fries">Do you want fries?</label>
            <select class="form-control" id="fries" name="fries">
                <option value="" selected>No Fries</option>
                <option value="french">French-Fries</option>    
            </select>
        </div>
        <div class="form-group">
            <label for="friesExtras">Do you want extras with your fries?</label><br>
            <input type="checkbox" name="friesExtras[]" id="friesExtras" value="ketchup"> Ketchup<br>
            <input type="checkbox" name="friesExtras[]" id="friesExtras" value="mayonnaise"> Mayonnaise<br>      
        </div><br>
        <div class="form-group">
            <label for="salad">Do you want a salad?</label>         
            <select class="form-control" id="salad" name="salad">
                <option value="" selected>No Salad</option>
                <option value="salad">Standard Salad</option>    
            </select>
        </div>
        <div class="form-group">
            <label for="saladExtras">Do you want extras with your salad?</label><br>  
            <input type="checkbox" name="saladExtras[]" id="saladExtras" value="vinaigrette"> Vinaigrette<br>
            <input type="checkbox" name="saladExtras[]" id="saladExtras" value="yogurt"> Yogurt Dressing<br>      
        </div><br> 
        <div class="form-group">
            <label for="drink">Do you want something to drink?</label>         
            <select class="form-control" id="drink" name="drink">
                <option value="" selected>Nothing to drink</option>
                <option value="cola">Cola</option>    
                <option value="lemonade">Lemonade</option>    
            </select>
        </div>     
    <button type="submit" class="btn btn-primary">Order</button>
    </form>
    HTML;
}

/**
 * @param OrderInterface $order
 */
function printOrderSummary(OrderInterface $order)
{
    $orderFinishTime = getOrderFinishTime($order->getPreparationTime());
    $price = number_format($order->getPrice() / 100, 2);
    echo <<<HTML
    <p>Thank you {$order->getCustomer()}.</p>
    <p>Your {$order->getName()} will be ready at {$orderFinishTime}.</p>
    <p>It has {$order->getKiloCalories()} kcal.</p>
    <p>Please pay {$price} €.</p>
    <a href="index.php">Another Order</a>
    HTML;
}

/**
 * Formats the preparation time for the Europe/Berlin timezone.
 *
 * @param int $preparationTime
 * @return string
 */
function getOrderFinishTime(int $preparationTime)
{
    $time = time() + $preparationTime;
    $date = DateTime::createFromFormat('U', (string) $time);
    $dateTimeZone = new DateTimeZone('Europe/Berlin');
    $date->setTimezone($dateTimeZone);

    return $date->format('H:i:s');
}
